/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>

extern WINDOW	*win_status;
extern buddywin_t	nullbwin;
extern faimconf_t	faimconf;
extern int	iamaway;

int	naim_login_phase3d_f(struct command_rx_struct *command) {
	char	*buddies = NULL,
		profile[] = "Today's mighty oak is just yesterday's nut that held its ground.";

	printf("Uploading buddy list and profile...\n");
	buddies = rmakebuddyar(&faimconf);
	assert(buddies != NULL);
	if (faimconf.profile == NULL)
		aim_send_login_phase3c_hi(buddies, profile);
	else
		aim_send_login_phase3c_hi(buddies, faimconf.profile);
	free(buddies);
	if (command != NULL) {
		aim_send_login_phase4_a();
		printf("You are now fully logged in.\n");
	}
	return(0);
}

int	naim_buddy_coming(struct command_rx_struct *command) {
	char	buf[11];
	int	len = command->data[10];

	bzero(buf, sizeof(buf));
	strncpy(buf, command->data+11, len);
	bcoming(buf);
	return(1);
}

int	naim_buddy_going(struct command_rx_struct *command) {
	char	buf[11];
	int	len = command->data[10];

	bzero(buf, sizeof(buf));
	strncpy(buf, command->data+11, len);
	bgoing(buf);
	return(1);
}

IMpack_t	defIMpacket;

int	naim_im_breakdown(struct command_rx_struct *command) {
	int	i = 20, tlvcnt = 0;
	unsigned int	msglen = 0;
	IMpack_t	IMpack;

	assert(command != NULL);
	assert(command->data[i] <= 10);	// I was guaranteed screennames
					// were always shorter than 10
					// characters; this code breaks if
					// that's untrue
	bzero(&IMpack, sizeof(IMpack));
	{
		int	a = 0, b = 0;

		for (; a < command->data[i]; a++)
			if (!isspace(command->data[i+1+a]))
				IMpack.screenname[b++] =
					command->data[i+1+a];
	}

	i += (int)command->data[i] + 3;
	tlvcnt = ((command->data[i++]) << 8) & 0xFF00;
	tlvcnt += (command->data[i++]) & 0x00FF;

// the following is adapted (ie, almost copied verbatim) from gtkfaim with
// permission from the authour
	IMpack.membersince = IMpack.onsince = IMpack.class =
		IMpack.idletime = 0;
	{
		int	curtlv = 0, tlv1 = 0;

		while (curtlv < tlvcnt) {
			if ((command->data[i] == 0x00)
				&& (command->data[i+1] == 0x01)) {
				if (tlv1)
					break;
				if (command->data[i+3] != 0x02)
					printf("userinfo: strange v(%x) for t(1)\n",
						command->data[i+3]);
				IMpack.class = (((command->data[i+4]) << 8) & 0xFF00)
					+ ((command->data[i+5]) & 0x00FF);
				i += (4 + command->data[i+3]);
				tlv1++;
			} else if ((command->data[i] == 0x00)
				&& (command->data[i+1] == 0x02)) {
				if (command->data[i+3] != 0x04)
					printf("userinfo: strange v(%x) for t(2)\n",
						command->data[i+3]);
				IMpack.membersince = (((command->data[i+4]) << 24) & 0xFF000000)
					+ (((command->data[i+5]) << 16) & 0x00FF0000)
					+ (((command->data[i+6]) << 8) & 0x0000FF00)
					+ (((command->data[i+7])) & 0x000000FF);
				i += (4 + command->data[i+3]);
			} else if ((command->data[i] == 0x00)
				&& (command->data[i+1] == 0x03)) {
				if (command->data[i+3] != 0x04)
					printf("userinfo: strange v(%x) for t(3)\n",
						command->data[i+3]);
				IMpack.onsince = (((command->data[i+4]) << 24) & 0xFF000000)
					+ (((command->data[i+5]) << 16) & 0x00FF0000)
					+ (((command->data[i+6]) << 8) & 0x0000FF00)
					+ (((command->data[i+7])) & 0x000000FF);
				i += (4 + command->data[i+3]);
			} else if ((command->data[i] == 0x00)
				&& (command->data[i+1] == 0x04)) {
				if (command->data[i+3] != 0x02)
					printf("userinfo: strange v(%x) for t(4)\n",
						command->data[i+3]);
				IMpack.idletime = (((command->data[i+4]) << 8) & 0xFF00)
					+ ((command->data[i+5]) & 0x00FF);
				i += (4 + command->data[i+3]);
			} else {
				printf("userinfo: unexpected TLV t(%02x%02x) l(%02x%02x)\n",
					command->data[i], command->data[i+1],
					command->data[i+2], command->data[i+3]);
				i += (4 + command->data[i+3]);
			}
			curtlv++;
		}
	}

/* detect if this is an auto-response or not -- from gtkfaim (ASF)       **
** auto-responses can be detected by the presence of a *second* TLV with **
** t(0004), but of zero length (and therefore no value portion)          */
	{
		struct aim_tlv_t	*tsttlv = NULL;

		tsttlv = aim_grabtlv((u_char *)&(command->data[i]));
		if (tsttlv->type == 0x04)
			IMpack.autoreply = TRUE;
		else
			IMpack.autoreply = FALSE;
		aim_freetlv(&tsttlv);
	}

	while (1) {
		if ((command->data[i] == 0x00)
			&& (command->data[i+1] == 0x00)
			&& (command->data[i+2] == 0x00)
			&& (command->data[i+3] == 0x00))
			break;
		else
			i++;
	}
	i -= 2;
	msglen = ((((unsigned int)command->data[i]) & 0xFF) << 8);
	msglen += ((unsigned int)command->data[i+1]) & 0xFF;
	msglen -= 4;
	i += 6;

	snprintf(IMpack.msg, sizeof(IMpack.msg), "%.*s", msglen,
		(command->data)+i);

	memcpy(&defIMpacket, &IMpack, sizeof(defIMpacket));
	naim_im_handle(&defIMpacket);
	return(0);
}

int	naim_im_handle(IMpack_t *pack) {
	buddywin_t	*bwin;
	time_t	t = 0;
	struct tm	*tmptr = NULL;
	ignorelist_t	*idiot = NULL;

	assert(pack != NULL);
	t = time(NULL);
	assert(t != 0);
	tmptr = localtime(&t);
	assert(tmptr != NULL);
	idiot = faimconf.idiots;
	while (idiot != NULL) {
		if (aimcmp(pack->screenname, idiot->screenname) == 0) {
			if ((t - idiot->informed) > 1200) {
				char	igbuf[1200];

				snprintf(igbuf, sizeof(igbuf), "You are"
					" on my ignore list: <font"
					" size=+1 absz=14>%s"
					"</font><br>\n"
					"Your IM was not received by me,"
					" it was ignored. You will receive"
					" no more IM's from me (at least"
					" for 20 minutes).<br>\n<font"
					" size=-1 absz=10>This IM"
					" is automatic.</font>",
					idiot->notes);
				naim_send_im_away(pack->screenname, igbuf);
				idiot->informed = t;
			}
			return(0);
		}
		idiot = idiot->next;
	}
	logim(pack->screenname, aim_connection.sn, pack->msg);
	if (faimconf.beeponim == 1)
		beep();
	if ((bwin = bgetwin(pack->screenname)) == NULL) {
		nullbwin.waiting = 1;
		if (pack->autoreply == FALSE) {
			hwprintf(win_status, IMWIN_TEXT_R,
				"[%02i:%02i:%02i] [<B>%s</B>] %s<br>",
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, pack->screenname,
				pack->msg);
			if (iamaway && (*faimconf.awaymsg != 0)
				&& ((t - nullbwin.informed) > 600)
				&& (aimcmp(pack->screenname,
				   aim_connection.sn) != 0)) {
				sendaway(pack->screenname);
				nullbwin.informed = t;
			}
		} else
			hwprintf(win_status, IMWIN_TEXT_R,
				"[%02i:%02i:%02i] [<B>%s</B> is away] %s<br>",
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, pack->screenname,
				pack->msg);
		bupdate();
	} else {
		bwin->keepafterso = bwin->waiting = 1;
		if (pack->autoreply == FALSE) {
			hwprintf(bwin->win, IMWIN_TEXT_R,
				"[%02i:%02i:%02i] <B>%s</B>: %s<br>", 
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, pack->screenname,
				pack->msg);
			if (iamaway && (*faimconf.awaymsg != 0)
				&& ((t - bwin->informed) > 600)
				&& (aimcmp(pack->screenname,
				   aim_connection.sn) != 0)) {
				sendaway(pack->screenname);
				bwin->informed = t;
			}
		} else
			hwprintf(bwin->win, IMWIN_TEXT_R,
				"[%02i:%02i:%02i] <B>%s</B> is away: %s<br>", 
				tmptr->tm_hour, tmptr->tm_min,
				tmptr->tm_sec, pack->screenname, 
				pack->msg);
		bupdate();
	}
	return(1);
}

#define SHIFTBYTE(value, which) \
	(((value) & 0xFF) << (8*which))

int	naim_userinfo_handler(struct command_rx_struct *command) {
	char	SN[11];
	u_long	snacid = 0, membersince = 0, onlinesince = 0;
	u_short	warnlevel = 0, idletime = 0, class = 0;
	int	i = 0, tlvcnt = 0, curtlv = 0;

	snacid = (command->data[6] << 24) & 0xFF000000;
	snacid += (command->data[7] << 16) & 0x00FF0000;
	snacid += (command->data[8] << 8) & 0x0000FF00;
	snacid += (command->data[9]) & 0x000000FF;
	{
		struct aim_snac_t	*snac = NULL;

		if ((snac = aim_remsnac(snacid)) != NULL) {
			free(snac->data);
			free(snac);
		} else {
			printf("SNAC packet not found in userinfo reply.\n");
			return(-1);
		}
	}

	bzero(SN, sizeof(SN));
	strncpy(SN, command->data+11, command->data[10]);
	i = 11 + command->data[10];
	warnlevel = ((command->data[i++]) << 8) & 0xFF00;
	warnlevel += (command->data[i++]) & 0x00FF;

	tlvcnt = ((command->data[i++]) << 8) & 0xFF00;
	tlvcnt += (command->data[i++]) & 0x00FF;

	for (curtlv = 0; curtlv < tlvcnt; curtlv++) {
		struct aim_tlv_t	*tlv = NULL;

		tlv = aim_grabtlv((u_char *)&(command->data[i]));
		switch (tlv->type) {
			case 0x0001:
				assert(tlv->length == 2);
				class = SHIFTBYTE(tlv->value[0], 1);
				class += SHIFTBYTE(tlv->value[1], 0);
				break;
			case 0x0002:
				assert(tlv->length == 4);
				membersince = SHIFTBYTE(tlv->value[0], 3);
				membersince += SHIFTBYTE(tlv->value[1], 2);
				membersince += SHIFTBYTE(tlv->value[2], 1);
				membersince += SHIFTBYTE(tlv->value[3], 0);
				break;
			case 0x0003:
				assert(tlv->length == 4);
				onlinesince = SHIFTBYTE(tlv->value[0], 3);
				onlinesince += SHIFTBYTE(tlv->value[1], 2);
				onlinesince += SHIFTBYTE(tlv->value[2], 1);
				onlinesince += SHIFTBYTE(tlv->value[3], 0);
				break;
			case 0x0004:
				assert(tlv->length == 2);
				idletime = SHIFTBYTE(tlv->value[0], 1);
				idletime += SHIFTBYTE(tlv->value[1], 0);
				break;
			default:
				printf("naim_userinfo_handler: received"
					" unknown TLV: t(%i), l(%i),"
					" v(%s)\n", tlv->type,
					tlv->length, tlv->value);
				break;
		}
		i += 4 + tlv->length;
		aim_freetlv(&tlv);
	}
	{
		time_t	t = 0;
		struct tm	*tmptr = NULL;

		printf("Received userinfo reply for %s:\n", SN);
		printf("  Warning level: %i%%\n", warnlevel/10);
		if ((t = membersince) != 0) {
			tmptr = localtime(&t);
			assert(tmptr != NULL);
			printf("   Member since: %02i-%02i-%04i at"
				" %02i:%02i\n", 1+tmptr->tm_mon,
				tmptr->tm_mday, 1900+tmptr->tm_year,
				tmptr->tm_hour, tmptr->tm_min);
		}
		if ((t = onlinesince) != 0) {
			tmptr = localtime(&t);
			assert(tmptr != NULL);
			printf("   Online since: %02i-%02i-%04i at"
				" %02i:%02i\n", 1+tmptr->tm_mon,
				tmptr->tm_mday, 1900+tmptr->tm_year,
				tmptr->tm_hour, tmptr->tm_min);
		}
		if (idletime > 0)
			printf("      Idle time: %i minutes\n", idletime);
	}

	if (i < command->commandlen) {
		struct aim_tlv_t	*tlv = NULL;

		if ((tlv = aim_grabtlv(command->data+i)) == NULL) {
			printf("naim_userinfo_handler: received bad TLV"
				" data\n");
			return(-1);
		}
		if (tlv->type == 0x0001)
			printf("Profile (%.*s):\n", tlv->length,
				tlv->value);
		else
			printf("naim_userinfo_handler: received unknown"
				" TLV: t(%i), l(%i), v(%s)\n", tlv->type,
				tlv->length, tlv->value);
		i += 4 + tlv->length;
		aim_freetlv(&tlv);
	}
	if (i < command->commandlen) {
		struct aim_tlv_t	*tlv = NULL;

		if ((tlv = aim_grabtlv(command->data+i)) == NULL) {
			printf("naim_userinfo_handler: received bad TLV"
				" data\n");
			return(-1);
		}
		hwprintf(win_status, STAT_TEXT, "%.*s<br><hr>",
			tlv->length, tlv->value);
		aim_freetlv(&tlv);
	}
	return(0);
}
