/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>

extern WINDOW	*win_buddy, *win_status;
extern faimconf_t	faimconf;

buddywin_t	nullbwin,
		*curbwin;

void	bsetup(void) {
	curbwin = &nullbwin;
	bzero(curbwin, sizeof(buddywin_t));
	strncpy(curbwin->winname, "Misc. Window", 12);
	curbwin->win = win_status;
	curbwin->next = curbwin;
	bupdate();
}

void	bupdate(void) {
	buddywin_t	*bwin = curbwin;

	assert(bwin != NULL);
	bwin->waiting = 0;
	wclear(win_buddy);
//	wprintf(win_buddy, BUDDY_WAITING, 1, "%.12s", "Window list ");
//	wprintf(win_buddy, BUDDY_WAITING, 0, "%.12s", "============");
	do {
		if (bwin == &nullbwin) {
			if (bwin->waiting)
				wprintf(win_buddy, BUDDY_WAITING, 1,
					"%12.12s", bwin->winname);
			else
				wprintf(win_buddy, BUDDY_TEXT, 1,
					"%12.12s", bwin->winname);
		} else {
			if (bwin->waiting)
				wprintf(win_buddy, BUDDY_WAITING, 1,
					"(%10.10s)", bwin->winname);
			else if (bwin->offline)
				wprintf(win_buddy, BUDDY_OFFLINE, 0,
					"<%10.10s>", bwin->winname);
			else if (bwin->notbuddy)
				wprintf(win_buddy, BUDDY_NONBUDDY, 1,
					"{%10.10s}", bwin->winname);
			else
				wprintf(win_buddy, BUDDY_TEXT, 1,
					"[%10.10s]", bwin->winname);
		}
	} while ((bwin = bwin->next) != curbwin);
}

int	bonline(char *buddy) {
	buddywin_t	*bwin = curbwin;

	assert(buddy != NULL);
	assert(bwin != NULL);
	if (aimcmp(buddy, bwin->winname) == 0)
		return(1);
	while ((bwin = bwin->next) != curbwin)
		if (aimcmp(buddy, bwin->winname) == 0)
			return(1);
	return(0);
}

buddywin_t	*bgetwin(char *buddy) {
	buddywin_t	*bwin = curbwin;

	assert(buddy != NULL);
	assert(bwin != NULL);
	if (aimcmp(buddy, bwin->winname) == 0)
		return(bwin);
	while ((bwin = bwin->next) != curbwin)
		if (aimcmp(buddy, bwin->winname) == 0)
			return(bwin);
	return(NULL);
}

void	bremove(buddywin_t *bwin) {
	assert(bwin != NULL);
	assert(bwin->win != NULL);
	assert(bwin->next != bwin);
	delwin(bwin->win);
	free(bwin);
}

#define NWIN(WIN)	newwin(faimconf.WIN.widthy, faimconf.WIN.widthx, \
	faimconf.WIN.starty, faimconf.WIN.startx);
void	bnewwin(char *buddy) {
	buddywin_t	*bwin = NULL;

	assert(buddy != NULL);
	bwin = calloc(1, sizeof(buddywin_t));
	assert(bwin != NULL);
	bwin->win = NWIN(wstatus);
	assert(bwin->win != NULL);
	winitwin(bwin->win, IMWIN_BGCOLOR);
	bzero(bwin->winname, 13);
	strncpy(bwin->winname, buddy, 10);
	bwin->keepafterso = bwin->notbuddy = bwin->waiting = 0;
	bwin->offline = 1;
	bwin->next = curbwin->next;
	curbwin->next = bwin;
}

void	bcoming(char *buddy) {
	time_t	t = 0;
	struct tm	*tmptr = NULL;
	buddywin_t	*bwin = NULL;
	buddylist_t	*blist = NULL;

	assert(buddy != NULL);
	t = time(NULL);
	assert(t != 0);
	tmptr = localtime(&t);
	assert(tmptr != NULL);

	if ((blist = rgetlist(&faimconf, buddy)) == NULL) {
		printf("Er, I just received a logon notice for %s, but I can't seem to find %s in my buddy list...\n", buddy, buddy);
		return;
	}
	if ((bwin = bgetwin(buddy)) == NULL) {
		bnewwin(buddy);
		bwin = bgetwin(buddy);
		assert(bwin != NULL);
	}
	if (bwin->offline == 1) {
		bwin->offline = 0;
		if (*(blist->notes) != 0)
			printf("%s %s (%s) is now online =)\n",
				blist->group, buddy, blist->notes);
		else
			printf("%s %s is now online =)\n", blist->group,
				buddy);
		wprintf(bwin->win, IMWIN_STATUS, 1,
			"[%02i:%02i:%02i] %s %s is now online =)\n",
			tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec, 
			blist->group, buddy);
	}
	bwin->notbuddy = 0;
	if (faimconf.beeponsignon == 1)
		beep();
	bupdate();
}

void	bgoing(char *buddy) {
	time_t	t = 0;
	struct tm	*tmptr = NULL;
	buddywin_t	*bwin = curbwin;
	buddylist_t	*blist = NULL;

	assert(buddy != NULL);
	assert(bwin != NULL);
	t = time(NULL);
	assert(t != 0);
	tmptr = localtime(&t);
	assert(tmptr != NULL);

	if ((blist = rgetlist(&faimconf, buddy)) == NULL)
		printf("Er, I just received a logoff notice for %s, but I can't seem to find %s in my buddy list...\n", buddy, buddy);
	else {
		if (*(blist->notes) != 0)
			printf("%s %s (%s) has just logged off :(\n",
				blist->group, buddy, blist->notes);
		else
			printf("%s %s has just logged off :(\n", 
				blist->group, buddy);
	}

	do {
		if (aimcmp(buddy, bwin->winname) == 0) {
			if (bwin->keepafterso == 1) {
				bwin->offline = 1;
				wprintf(bwin->win, IMWIN_STATUS, 1,
 					"[%02i:%02i:%02i] %s %s has just"
					" logged off :(\n",
					tmptr->tm_hour, tmptr->tm_min,
					tmptr->tm_sec, blist->group, 
					buddy);
				bupdate();
				if (faimconf.beeponsignon == 1)
					beep();
			} else {
				buddywin_t	*bbefore = bwin->next,
						*bnext = bwin->next;

				while (bbefore->next != bwin)
					bbefore = bbefore->next;
				if (curbwin == bwin)
					curbwin = bnext;
				bremove(bwin);
				bbefore->next = bnext;
				bupdate();
				touchwin(curbwin->win);
			}
			return;
		} 
	} while ((bwin = bwin->next) != curbwin);
	printf("Uhm, I kinda couldn't find %s in your active buddy list."
		" That's probably bad.\n", buddy);
	bupdate();
}
