/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>

extern WINDOW	*win_input;
extern buddywin_t	*curbwin, nullbwin;
extern faimconf_t	faimconf;
extern int	iamaway;

void	handleline(char *line) {
	char	*SN, *msg;
	time_t	t = 0;
	struct tm	*tmptr = NULL;

	t = time(NULL);
	assert(t != 0);
	tmptr = localtime(&t);
	assert(tmptr != NULL);
	if (curbwin == &nullbwin) {
		SN = line;
		msg = firstwhite(line);
		if ((msg == NULL) || (*msg == 0)) {
			if (*SN == '/') {
				SN++;
				if (strlen(SN) > 10) {
					printf("Screen names must be 10 characters or shorter.\n");
				} else {
				  if (bgetwin(SN) != NULL)
					printf("There is already a window open for %s.\n", SN);
				  else {
					buddywin_t	*bwin = NULL;

					printf("Opened window for %s.\n", SN);
					bnewwin(SN);
					bwin = bgetwin(SN);
					assert(bwin != NULL);
					bwin->offline = 0;
					bwin->notbuddy = 1;
					wprintf(bwin->win, IMWIN_STATUS, 1, "Non-buddy window created.\n");
					bupdate();
				  }
				}
			} else {
				if (strlen(SN) > 10)
					printf("Screen names must be 10 characters or shorter.\n");
				else
					aim_getinfo(SN);
			}
			return;
		}
		if (strlen(SN) > 10) {
			printf("Screen names must be 10 characters or shorter.\n");
			return;
		}
		hwprintf(curbwin->win, IMWIN_TEXT_U,
			"[%02i:%02i:%02i] -> *<B>%s</B>* %s<br>",
			tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec, SN,
			msg);
	} else {
		SN = curbwin->winname;
		msg = line;
		hwprintf(curbwin->win, IMWIN_TEXT_U,
			"[%02i:%02i:%02i] <B>%s</B>: %s<br>",
			tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec,
			aim_connection.sn, line);
		curbwin->keepafterso = 1;
	}
	assert(SN != NULL);
	assert(*SN != 0);
	assert(msg != NULL);
	assert(*msg != 0);
	logim(aim_connection.sn, SN, msg);
	naim_send_im(SN, msg);
}

void	gotkey(int c) {
#ifndef NO_HISTORY
	HISTORY_STATE	*hstate = NULL;
	static int	histpos;
#endif
	static int	bufloc;
	static char	buf[1024], gotesc, inbold, initalic;

	updateidletime();
	if (c == 0) {
		bzero(buf, 1024);
		bufloc = initalic = inbold = gotesc = 0;
#ifndef NO_HISTORY
		histpos = 0;
#endif
		return;
	}
	assert(curbwin != NULL);
	if (bufloc > 900) {
		printf("Input buffer exceeded.\n");
		return;
	}
	if (c == 27) {
		if (!gotesc) {
			gotesc = 1;
			printf("Press Escape again to exit.\n");
		} else {
			printf("Logging off...\n");
			aim_logoff();
			wshutitdown();
			exit(0);
		}
		return;
	}
	gotesc = 0;
	switch(c) {
		case ERR:
			break;
		case KEY_UP:
#ifndef NO_HISTORY
			hstate = history_get_history_state();
			assert(hstate != NULL);
			if (hstate->entries == NULL)
				break;
			if (histpos > 0) {
				histpos--;
				assert(hstate->entries[histpos] != NULL);
				bzero(buf, 1024);
				strncpy(buf,
					(hstate->entries[histpos])->line, 1023);
				wclear(win_input);
				wprintf(win_input, INPUT_TEXT, 1, "%s", buf);
				bufloc = strlen(buf);
			}
#endif
			break;
		case KEY_DOWN:
#ifndef NO_HISTORY
			hstate = history_get_history_state();
			assert(hstate != NULL);
			if (hstate->entries == NULL)
				break;
			if (histpos < hstate->length) {
				histpos++;
				bzero(buf, 1024);
				if (hstate->entries[histpos] != NULL)
					strncpy(buf,
						(hstate->entries[histpos])->line, 1023);
				wclear(win_input);
				wprintf(win_input, INPUT_TEXT, 1, "%s", buf);
				bufloc = strlen(buf);
			}
#endif
			break;
		case KEY_LEFT:
			if (bufloc > 0) {
				waddstr(win_input, "\b");
				bufloc--;
			}
			break;
		case KEY_RIGHT:
			if (buf[bufloc] != 0) {
				wprintf(win_input, INPUT_TEXT, 1, "%c",
					buf[bufloc]);
				bufloc++;
			}
			break;
		case '\t':
			curbwin = curbwin->next;
			bupdate();
			touchwin(curbwin->win);
			break;
		case '\b': case KEY_BACKSPACE:
			if (bufloc < 1)
				break;
			buf[--bufloc] = 0;
			waddstr(win_input, "\b \b");
			break;
		case '\r': case '\n':
			assert(((bufloc == 0) && (*buf == 0))
				|| ((bufloc > 0) && (*buf != 0)));
			if (bufloc == 0)
				break;
#ifndef NO_HISTORY
			add_history(buf);
#endif
			handleline(buf);
#ifndef NO_HISTORY
			hstate = history_get_history_state();
			assert(hstate != NULL);
			histpos = hstate->length;
#endif
			bzero(buf, 1024);
			inbold = initalic = bufloc = 0;
			wclear(win_input);
			break;
		case '<':
			buf[bufloc++] = '&';
			buf[bufloc++] = 'l';
			buf[bufloc++] = 't';
			buf[bufloc++] = ';';
			wprintf(win_input, INPUT_TEXT_ALT, 1, "&lt;");
			break;
		case '>':
			buf[bufloc++] = '&';
			buf[bufloc++] = 'g';
			buf[bufloc++] = 't';
			buf[bufloc++] = ';';
			wprintf(win_input, INPUT_TEXT_ALT, 1, "&gt;");
			break;
		case '&':
			buf[bufloc++] = '&';
			buf[bufloc++] = 'a';
			buf[bufloc++] = 'm';
			buf[bufloc++] = 'p';
			buf[bufloc++] = ';';
			wprintf(win_input, INPUT_TEXT_ALT, 1, "&amp;");
			break;
		case KEY_F(1): case 11:
			printf("naim, n's ncurses-based FAIM frontend.\n");
			printf("The window listed at the top of the \"Window list\"\n");
			printf("window is the current window. Use the Tab key to\n");
			printf("cycle through the listed windows.\n");
			printf("While in the status window, you can type:\n");
			printf("  <SN>               Request information about <SN>\n");
			printf("  <SN> <message>     Send an IM to <SN>\n");
			printf("  /<SN>              Open a new window for <SN>\n");
			printf("While in a buddy window, anything you type will be\n");
			printf("sent to the buddy as an IM.\n");
			printf("F2 will insert either <B> or </B> into your input\n");
			printf("buffer. F3 does the same for <I>. F10 will close the\n");
			printf("current buddy window.\n");
			printf("See http://users.n.ml.org/nc/naim/ for more info.\n");
			break;
		case KEY_F(2): case 2:
			buf[bufloc++] = '<';
			if (inbold) {
				buf[bufloc++] = '/';
				wprintf(win_input, INPUT_TEXT_ALT, 1, "</B>");
			} else
				wprintf(win_input, INPUT_TEXT_ALT, 1, "<B>");
			buf[bufloc++] = 'B';
			buf[bufloc++] = '>';
			inbold = (inbold == 0);
			break;
		case KEY_F(3): case 22:
			buf[bufloc++] = '<';
			if (initalic) {
				buf[bufloc++] = '/';
				wprintf(win_input, INPUT_TEXT_ALT, 1,  "</I>");
			} else
				wprintf(win_input, INPUT_TEXT_ALT, 1, "<I>");
			buf[bufloc++] = 'I';
			buf[bufloc++] = '>';
			initalic = (initalic == 0);
			break;
		case KEY_F(4):
			printf("F4 is currently unused.\n");
			break;
		case KEY_F(5):
//			printf("F5 is currently unused.\n");
			buf[bufloc++] = '&';
			wprintf(win_input, INPUT_TEXT_ALT, 1, "&");
			break;
		case KEY_F(6): case 12:
			wclear(curbwin->win);
			break;
		case KEY_F(7): case 18: {
			char	buf[256];

			killconf(&faimconf);
			snprintf(buf, sizeof(buf), "%s/.faimrc",
				getenv("HOME"));
			printf("Re-reading %s\n", buf);
			naim_read_config(buf);
			wsetp();
			naim_login_phase3d_f(NULL);
			break;
		}
		case KEY_F(8): case 14: {
			buddywin_t	*bwin = curbwin;

			while (((bwin = bwin->next) != curbwin)
				&& (bwin->waiting == 0));
			if (bwin != curbwin) {
				curbwin = bwin;
				bupdate();
				touchwin(curbwin->win);
			}
			break;
		}
		case KEY_F(9): case 1:
			if (iamaway)
				unsetaway();
			else
				setaway();
			break;
		case KEY_F(10): case 23: {
			buddywin_t	*bwin = curbwin, *btmp = curbwin;

			if (curbwin == &nullbwin) {
				printf("You cannot close the status window.\n");
				break;
			}
			printf("Closed buddy window %s.\n",
				curbwin->winname);
			bwin = curbwin->next;
			while (btmp->next != curbwin)
				btmp = btmp->next;
			bremove(curbwin);
			btmp->next = curbwin = bwin;
			bupdate();
			touchwin(curbwin->win);
			break;
		}
		case KEY_F(11):
			buf[bufloc++] = '<';
			wprintf(win_input, INPUT_TEXT_ALT, 1, "<");
			break;
		case KEY_F(12):
			buf[bufloc++] = '>';
			wprintf(win_input, INPUT_TEXT_ALT, 1, ">");
			break;
		default:
			if (isprint(c)) {
				buf[bufloc++] = c;
				wprintf(win_input, INPUT_TEXT, 1, "%c", c);
				break;
			}
	}
}
