/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
**
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>

extern buddywin_t	*curbwin;
extern faimconf_t	faimconf;

#define CHECKTAG(tag)	(strcasecmp(tagbase, (tag)) == 0) {
#define CHECKAMP(tag)	(strcasecmp(tagbuf, (tag)) == 0) {
#define ELIF		} else if

unsigned long	parsehtml_tag(WINDOW *window, char *text, char *inbold,
			char *initalic, char *inunderline) {
	char	tagbuf[20], argbuf[1024], *textsave = text, *tagbase;
	int	tagpos = 0;

	bzero(tagbuf, sizeof(tagbuf));
	bzero(argbuf, sizeof(argbuf));

	while (isspace(*text))
		text++;
	while ((*text) && (!isspace(*text)) && (*text != '>')) {
		if (tagpos < sizeof(tagbuf))
			tagbuf[tagpos++] = *text;
		text++;
	}
	while (isspace(*text))
		text++;
	tagpos = 0;
	while ((*text) && (*text != '>')) {
		if (tagpos < sizeof(argbuf))
			argbuf[tagpos++] = *text;
		text++;
	}
	tagbase = tagbuf;
	if (tagbuf[0] == '/')
		tagbase++;
	wattrset(window, A_BOLD | COLOR_PAIR(IMWIN_STATUS));
	if CHECKTAG("B")
		*inbold = (*inbold == 0);
	ELIF CHECKTAG("I")
		*initalic = (*initalic == 0);
	ELIF CHECKTAG("U")
		*inunderline = (*inunderline == 0);
	ELIF CHECKTAG("A")
		if (tagbuf[0] == '/')
			waddstr(window, "]");
		else {
			waddstr(window, "[LINK:");
			waddstr(window, argbuf);
			waddstr(window, ":");
		}
	ELIF CHECKTAG("IMG")
		waddstr(window, "[IMAGE:");
		waddstr(window, argbuf);
		waddstr(window, "]");
	ELIF CHECKTAG("BR")
		waddstr(window, "\n");
	ELIF CHECKTAG("HR")
		waddstr(window, "----------------\n");
	ELIF CHECKTAG("FONT")
		;
	ELIF CHECKTAG("PRE")
		;
	ELIF CHECKTAG("P")
		;
	ELIF CHECKTAG("HTML")
		;
	ELIF CHECKTAG("BODY")
		;
	} else
		return(0);
	return((unsigned long)(text-textsave+1));
}

unsigned long	parsehtml_amp(WINDOW *window, char *text) {
	char	tagbuf[20], *textsave = text;
	int	tagpos = 0;

	bzero(tagbuf, sizeof(tagbuf));

	while (isspace(*text))
		text++;
	while ((*text) && (!isspace(*text)) && (*text != ';')
		&& (*text != '\n')) {
		if (tagpos < sizeof(tagbuf))
			tagbuf[tagpos++] = *text;
		else
			return(0);
		text++;
	}
	if (*text != ';')
		return(0);
	wattrset(window, A_BOLD | COLOR_PAIR(IMWIN_STATUS));
	if CHECKAMP("NBSP")
		waddstr(window, " ");
	ELIF CHECKAMP("AMP")
		waddstr(window, "&");
	ELIF CHECKAMP("LT")
		waddstr(window, "<");
	ELIF CHECKAMP("GT")
		waddstr(window, ">");
	} else
		return(0);
	return((unsigned long)(text-textsave+1));
}

unsigned long	parsehtml(WINDOW *w, char *a, char *b, char *c, char *d) {
	if (*a == '<')
		return(parsehtml_tag(w, a+1, b, c, d));
	else if (*a == '&')
		return(parsehtml_amp(w, a+1));
	else
		return(0);
}

int	hwprintf(WINDOW *window, int pair, const char *format, ...) {
	char	buf[10240], inbold = 0, initalic = 0, inunderline = 0;
	va_list	msg;
	size_t	len = 0;
	int	pos = -1, lines = 0;

	assert(window != NULL);
	assert(format != NULL);

	bzero(buf, sizeof(buf));

	va_start(msg, format);
	len = vsnprintf(buf, sizeof(buf), format, msg);
	va_end(msg);

	while (++pos < len) {
		if ((buf[pos] == '<') || (buf[pos] == '&')) {
			unsigned long	skiplen = 0;

			if ((skiplen = parsehtml(window, buf+pos, &inbold,
				&initalic, &inunderline)) == 0) {
				wattrset(window, A_BOLD |
					COLOR_PAIR(IMWIN_STATUS));
				waddnstr(window, buf+pos, 1);
				continue;
			}
			pos += skiplen;
			continue;
		} else {
			if (buf[pos] == '\r')
				continue;
			if (buf[pos] == '\n') {
				waddstr(window, " ");
				continue;
			}
			wattrset(window, (inbold?A_BOLD:0)
				| (inunderline?A_UNDERLINE:0)
				| (initalic?A_REVERSE:0) | COLOR_PAIR(pair));
			if (isprint(buf[pos]))
				waddnstr(window, buf+pos, 1);
			else
				waddnstr(window, ".", 1);
			wattrset(window, 0);
			continue;
		}
	}
	return(lines);
}
