/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
** 
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>

extern WINDOW	*win_status, *win_input, *win_buddy;
extern buddywin_t	*curbwin;
extern faimconf_t	faimconf;
extern int	iamaway, wsetup_called;
time_t	idletime = 0;

void	updateidletime(void) {
	time_t	t = time(NULL);

	assert(t != 0);
	idletime = t;
}

int	main(int argc, char **args) {
	struct timeval	timeout;
	time_t	t = 0;
	rxcallback_t	faimtest_callbacks[] = {
		naim_im_breakdown,
			/* incoming IM */
		naim_buddy_coming,
			/* oncoming buddy */
		naim_buddy_going, /* offgoing buddy */
		NULL,	/* last IM was missed 1 */
		NULL,	/* last IM was missed 2 */
		NULL,	/* login phase 4 packet C command 1*/
		NULL,	/* login phase 4 packet C command 2 */
		NULL,	/* login phase 2, first resp */
		NULL,	/* login phase 2, second resp */
		NULL,	/* login phase 3 packet B */
		NULL,	/* login phase 3D packet A */
		NULL,	/* login phase 3D packet B */
		NULL,	/* login phase 3D packet C */
		NULL,	/* login phase 3D packet D */
		NULL,	/* login phase 3D packet E */
		naim_login_phase3d_f,
			/* login phase 3C packet F */
		NULL,	/* last command bad */
		NULL,	/* missed some messages */
		NULL,	/* completely unknown command */
		naim_userinfo_handler,
			/* User Info Response */
		NULL,	/* User Search by Address response */
		NULL,	/* User Search by Name response */
		0x00
	};
	fd_set	fds, errfds;
	char	buf[256];

	initscr();
	wsetup_called = 2;
	snprintf(buf, sizeof(buf), "%s/.faimrc", getenv("HOME"));
	printf("Reading %s\n", buf);
	if (naim_read_config(buf) == 0) {
		wshutitdown();
		printf("Error %i reading %s: %s\n", errno, buf, 
			strerror(errno));
		exit(-1);
	}
	endwin();
	wsetup_called = 0;
	wsetup();
	bsetup();
	gotkey(0);	// initialize gotkey() buffer
	wcols(&faimconf);
	aim_register_callbacks(faimtest_callbacks);
	printf("Logging on as %s\n", faimconf.screenname);
	wnoutrefresh(curbwin->win);
	doupdate();
	if (aim_login(faimconf.screenname, faimconf.password) != 0) {
		wshutitdown();
		printf("FAIM was unable to log on to AIM.\n");
		exit(1);
	}
	if (faimconf.nolog == 0) {
		if (*(faimconf.logfile) == 0)
			strncpy(faimconf.logfile, "naim.log",
				sizeof(faimconf.logfile)-1);
		printf("Opening log file (%s)\n", faimconf.logfile);
		logim(NULL, NULL, faimconf.logfile);
	} else if (*(faimconf.logfile) != 0)
		printf("You have logging disabled, but you specified a log file.\n");
	printf("Contact Daniel Reed <n@ml.org> if you'd like\n");
	printf("information about naim.\n");
	wnoutrefresh(curbwin->win);
	doupdate();

	timeout.tv_sec = 40;
	timeout.tv_usec = 0;

	FD_ZERO(&errfds);
	FD_SET(aim_connection.fd, &errfds);
	FD_ZERO(&fds);
	FD_SET(aim_connection.fd, &fds);
	FD_SET(STDIN_FILENO, &fds);

	while ((select((aim_connection.fd)+1, &fds, NULL, &errfds,
		&timeout) >= 0)) {
		time_t	now = time(NULL);

		if ((faimconf.autoaway != 0) && ((now - idletime)
			>= 60*(faimconf.autoaway)) && (iamaway == 0)) {
			printf("You have been idle for more than %i"
				" minutes.\n", faimconf.autoaway);
			setaway();
		}
		if ((now - t) >= 60) {
			if (faimconf.tprint != 0) {
				struct tm	*tptr;

				tptr = localtime(&now);
				assert(tptr != NULL);
				if ((tptr->tm_min)%(faimconf.tprint) == 0) {
					t = now;
					hwprintf(win_status, IMWIN_TEXT_U,
						"<I>----</I> "
						"[<B>%02i</B>:<B>%02i</B>]"
						" <I>----</I><br>",
						tptr->tm_hour, tptr->tm_min);
				}
			}
		}
		if (FD_ISSET(aim_connection.fd, &errfds)) {
			printf("Error detected on network socket :(\n");
			break;
		}
		if (FD_ISSET(STDIN_FILENO, &fds))
			gotkey(getch());
		if (FD_ISSET(aim_connection.fd, &fds)) {
			aim_get_command();
			aim_rxdispatch();
		}

		timeout.tv_sec = 0;
		timeout.tv_usec = 1000;
		FD_ZERO(&fds);
		FD_SET(aim_connection.fd, &fds);
		FD_SET(STDIN_FILENO, &fds);
		if (select((aim_connection.fd)+1, &fds, NULL, NULL,
			&timeout) == 0) {
			wnoutrefresh(win_buddy);
			wnoutrefresh(curbwin->win);
			wnoutrefresh(win_input);
			doupdate();
		}
		timeout.tv_sec = 40;
		timeout.tv_usec = 0;

		FD_ZERO(&errfds);
		FD_SET(aim_connection.fd, &errfds);
		FD_ZERO(&fds);
		FD_SET(aim_connection.fd, &fds);
		FD_SET(STDIN_FILENO, &fds);
	}
	printf("AIM just decided we didn't need to be here anymore, sleeping 5, then dying...\n");
	sleep(5);
	aim_logoff();
	wshutitdown();
	exit(0);
}
