/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
**
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>
#include <stdio.h>

#ifdef HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif

faimconf_t	faimconf;

void	killconf(faimconf_t *conf) {
	buddylist_t	*buddy = NULL;
	ignorelist_t	*idiot = NULL;

	assert(conf != NULL);
	buddy = conf->buddies;
	idiot = conf->idiots;

	while (buddy != NULL) {
		buddylist_t	*bnext = buddy->next;

		free(buddy);
		buddy = bnext;
	}
	while (idiot != NULL) {
		ignorelist_t	*inext = idiot->next;

		free(idiot);
		idiot = inext;
	}
	if (faimconf.profile != NULL)
		free(faimconf.profile);
	bzero(conf, sizeof(faimconf_t));
}

void	raddbuddy(faimconf_t *conf, char *screenname, char *group,
	char *notes) {
	buddylist_t	*buddy = NULL;

	assert(conf != NULL);
	assert(screenname != NULL);
	assert(group != NULL);
	buddy = calloc(1, sizeof(buddylist_t));
	assert(buddy != NULL);
	strncpy(buddy->screenname, screenname,
		sizeof(buddy->screenname)-1);
	if (notes != NULL)
		strncpy(buddy->notes, notes, sizeof(buddy->notes)-1);
	if (group != NULL)
		strncpy(buddy->group, group, sizeof(buddy->group)-1);
	buddy->next = conf->buddies;
	conf->buddies = buddy;
}

void	raddidiot(faimconf_t *conf, char *screenname, char *notes) {
	ignorelist_t	*idiot = NULL;

	assert(conf != NULL);
	assert(screenname != NULL);
	idiot = calloc(1, sizeof(ignorelist_t));
	assert(idiot != NULL);
	strncpy(idiot->screenname, screenname,
		sizeof(idiot->screenname)-1);
	if (notes != NULL)
		strncpy(idiot->notes, notes, sizeof(idiot->notes)-1);
	else
		strncpy(idiot->notes, "There is no reason, you're just"
			" ignored!", sizeof(idiot->notes)-1);
	idiot->next = conf->idiots;
	conf->idiots = idiot;
}

buddylist_t	*rgetlist(faimconf_t *conf, char *screenname) {
	buddylist_t	*blist = NULL;

	assert(conf != NULL);
	assert(screenname != NULL);
	blist = conf->buddies;
	assert(blist != NULL);
	do {
		if (aimcmp(blist->screenname, screenname) == 0)
			return(blist);
	} while ((blist = blist->next) != NULL);
	return(NULL);
}

char	*rmakebuddyar(faimconf_t *conf) {
	buddylist_t	*buddy = NULL;
	size_t	len = 0;
	char	*buf = NULL;

	assert(conf != NULL);
	if ((buddy = conf->buddies) == NULL)
		return(NULL);
	do {
		len += strlen(buddy->screenname)+1;
	} while ((buddy = buddy->next) != NULL);
	buf = calloc(1, len+1);
	assert(buf != NULL);

	buddy = conf->buddies;
	do {
		strcat(buf, buddy->screenname);
		strcat(buf, "&");
	} while ((buddy = buddy->next) != NULL);

	return(buf);
}

static char	*bname(char *s) {
	register char	*p;

	assert(s != NULL);
	p = s + strlen(s);
	while ((p > s) && (*p != '/'))
		p--;
	return((*p=='/')?(p+1):p);
}

int	naim_eval(char *str) {
	assert(str != NULL);
	if (*str == 0)
		return(0);
	if (strncasecmp(str, "LINES", 5) == 0) {
		str += 5;
		while ((*str != 0) && isspace(*str))
			str++;
		if (*str == 0)
			return(LINES);
		if (*str == '-') {
			str++;
			while ((*str != 0) && isspace(*str))
				str++;
			return(LINES - atoi(str));
		}
		while ((*str != 0) && isspace(*str))
			str++;
		return(atoi(str));
	}
	if (strncasecmp(str, "COLS", 4) == 0) {
		str += 4;
		while ((*str != 0) && isspace(*str))
			str++;
		if (*str == 0)
			return(COLS);
		if (*str == '-') {
			str++;
			while ((*str != 0) && isspace(*str))
				str++;
			return(COLS - atoi(str));
		}
		while ((*str != 0) && isspace(*str))
			str++;
		return(atoi(str));
	}
	return(atoi(str));
}

int	naim_read_config(char *faimrcfile) {
	FILE	*faimrc = NULL;
	char	buf[256],	// gtkFAIM's max is 255, why differ?
		*prefix = NULL, *keyword = NULL, *arg = NULL;
	int	linenum = 0;
	struct stat	statbuf;

	bzero(&faimconf, sizeof(faimconf));
	faimconf.tprint = 30;
	if ((faimrc = fopen(faimrcfile, "r")) == NULL)
		return(0);
	while (!feof(faimrc)) {
		fgets(buf, 255, faimrc);
		if (feof(faimrc))
			break;
		linenum++;
		if ((*buf == 0) || (*buf == '#'))
			continue;
		if (((arg = firstwhite(buf)) == NULL) || (*arg == 0))
			continue;
		prefix = atom(buf);
		while ((arg[strlen(arg)-1] == '\n')
			|| (arg[strlen(arg)-1] == '\r'))
			arg[strlen(arg)-1] = 0;
		if (((keyword = firstatom(prefix, ".")) == NULL)
			|| (*keyword == 0))
			keyword = prefix;
		else
			if (strcasecmp(prefix, "naim") != 0)
				continue;

		if (strcasecmp(keyword, "screenname") == 0) {
			char	*garbage = firstwhite(arg);

			if (garbage != NULL) {
				printf("%s[%i]: screenname must have one"
					" argument, yet you have extra"
					" (%s, %s)\n", bname(faimrcfile),
					linenum, arg, garbage);
				continue;
			}
			strncpy(faimconf.screenname, arg,
				sizeof(faimconf.screenname)-1);
			printf("Set screen name to %s.\n",
				faimconf.screenname);
			continue;
		}
		if (strcasecmp(keyword, "password") == 0) {
			char	*garbage = firstwhite(arg);

			if (garbage != NULL) {
				printf("%s[%i]: password must have one"
					" argument, yet you have extra"
					" (%s, %s)\n", bname(faimrcfile),
					linenum, arg, garbage);
				continue;
			}
			strncpy(faimconf.password, arg,
				sizeof(faimconf.password)-1);
			continue;
		}
		if (strcasecmp(keyword, "awaymsg") == 0) {
			strncpy(faimconf.awaymsg, arg,
				sizeof(faimconf.awaymsg)-1);
			printf("Set away message to \"%s\".\n",
				faimconf.awaymsg);
			continue;
		}
		if (strcasecmp(keyword, "logfile") == 0) {
			strncpy(faimconf.logfile, arg,
				sizeof(faimconf.logfile)-1);
			printf("Logging IM's to \"%s\".\n",
				faimconf.logfile);
			continue;
		}
		if (strcasecmp(keyword, "buddy") == 0) {
			char	*group = NULL, *notes = NULL;

			if (((group = firstwhite(arg)) == NULL)
				|| (*group == 0))
				raddbuddy(&faimconf, atom(arg), "Buddy",
					NULL);
			else if (((notes = firstwhite(group)) == NULL)
				|| (*notes == 0))
				raddbuddy(&faimconf, atom(arg),
					atom(group), NULL);
			else
				raddbuddy(&faimconf, atom(arg),
					atom(group), notes);
			continue;
		}
		if (strcasecmp(keyword, "ignore") == 0) {
			char	*notes = NULL;

			if (((notes = firstwhite(arg)) == NULL)
				|| (*notes == 0))
				raddidiot(&faimconf, arg, NULL);
			else
				raddidiot(&faimconf, arg, notes);
			continue;
		}
		if (strcasecmp(keyword, "beeponim") == 0) {
			faimconf.beeponim = atoi(arg);
			continue;
		}
		if (strcasecmp(keyword, "beeponsignon") == 0) {
			faimconf.beeponsignon = atoi(arg);
			continue;
		}
		if (strcasecmp(keyword, "autoaway") == 0) {
			faimconf.autoaway = atoi(arg);
			continue;
		}
		if (strcasecmp(keyword, "nolog") == 0) {
			faimconf.nolog = atoi(arg);
			continue;
		}
		if (strcasecmp(keyword, "tprint") == 0) {
			faimconf.tprint = atoi(arg);
			continue;
		}
		if (strcasecmp(keyword, "profile") == 0) {
			char	*tempprof = NULL;
			size_t	len = 0;
			int	proffd = 0;

			if (faimconf.profile != NULL) {
				printf("%s[%i]: profile line"
					" specified, but I've already"
					" read in a profile!\n",
					bname(faimrcfile), linenum);
				continue;
			}
			if (stat(arg, &statbuf) == -1)
				statbuf.st_mode = 0;
			if ((len = statbuf.st_size) < 1) {
				printf("%s[%i]: profile file too small\n",
					bname(faimrcfile), linenum);
				continue;
			}
			if ((tempprof = calloc(1, len)) == NULL) {
				printf("%s[%i]: unable to allocate %i"
					" bytes for profile file: %s\n",
					bname(faimrcfile), linenum,
					len, strerror(errno));
				continue;
			}
			if ((proffd = open(arg, O_RDONLY)) < 0) {
				printf("%s[%i]: unable to open profile"
					" file: %s\n",
					bname(faimrcfile), linenum,
					strerror(errno));
				free(tempprof);
				continue;
			}
			if (read(proffd, tempprof, len) < len) {
				printf("%s[%i]: error reading in profile:"
					" %s\n", bname(faimrcfile),
					linenum, strerror(errno));
				free(tempprof);
				continue;
			}
			faimconf.profile = tempprof;
			close(proffd);
			printf("Read profile from \"%s\".\n", arg);
			continue;
		}
		if (strcasecmp(keyword, "color") == 0) {
			char	*fore = NULL, *back = NULL;

			if (((fore = firstwhite(arg)) == NULL)
				|| (*fore == 0)) {
				printf("%s[%i]: color keyword given without "
					"foreground color.\n",
					bname(faimrcfile), linenum);
				continue;
			}
			if (((back = firstwhite(fore)) == NULL)
				|| (*back == 0)) {
				printf("%s[%i]: color keyword given without "
					"background color.\n",
					bname(faimrcfile), linenum);
				continue;
			}
			faimconf.colors[atoi(arg)].fore = atoi(fore);
			faimconf.colors[atoi(arg)].back = atoi(back);
			continue;
		}
		if (strcasecmp(keyword, "window") == 0) {
			char	*startx = NULL, *starty = NULL,
				*widthx = NULL, *widthy = NULL;

			if (((startx = firstwhite(arg)) == NULL)
				|| (*startx == 0)) {
				printf("%s[%i]: window keyword given without "
					"starting x position.\n",
					bname(faimrcfile), linenum);
				continue;
			}
			if (((starty = firstwhite(startx)) == NULL)
				|| (*starty == 0)) {
				printf("%s[%i]: window keyword given without "
					"starting y position.\n",
					bname(faimrcfile), linenum);
				continue;
			}
			if (((widthx = firstwhite(starty)) == NULL)
				|| (*widthx == 0)) {
				printf("%s[%i]: window keyword given without "
					"x width.\n",
					bname(faimrcfile), linenum);
				continue;
			}
			if (((widthy = firstwhite(widthx)) == NULL)
				|| (*widthy == 0)) {
				printf("%s[%i]: window keyword given without "
					"y width.\n",
					bname(faimrcfile), linenum);
				continue;
			}

			if (strcasecmp(arg, "STATUS") == 0) {
				faimconf.wstatus.startx = naim_eval(startx);
				faimconf.wstatus.starty = naim_eval(starty);
				faimconf.wstatus.widthx = naim_eval(widthx);
				faimconf.wstatus.widthy = naim_eval(widthy);
			} else if (strcasecmp(arg, "INPUT") == 0) {
				faimconf.winput.startx = naim_eval(startx);
				faimconf.winput.starty = naim_eval(starty);
				faimconf.winput.widthx = naim_eval(widthx);
				faimconf.winput.widthy = naim_eval(widthy);
			} else if (strcasecmp(arg, "BUDDY") == 0) {
				faimconf.wbuddy.startx = naim_eval(startx);
				faimconf.wbuddy.starty = naim_eval(starty);
				faimconf.wbuddy.widthx = naim_eval(widthx);
				faimconf.wbuddy.widthy = naim_eval(widthy);
			} else {
				printf("%s[%i]: window keyword given with an "
					"invalid window name (%s).\n", 
					bname(faimrcfile), linenum, arg);
				continue;
			}
			printf("Creating window %s at (%i, %i) and"
				" extending %ix%i.\n", arg, 
				naim_eval(startx), naim_eval(starty),
				naim_eval(widthx), naim_eval(widthy));
			continue;
		}

		printf("%s[%i]: ignored keyword %s (%s)\n",
			bname(faimrcfile), linenum, keyword, arg);
	}
	return(1);
}
