/*  _ __   __ _ ___ __  __
** | '_ \ / _` |_ _|  \/  | naim
** | | | | (_| || || |\/| | Copyright 1998, 1999 Daniel Reed <n@ml.org>
** |_| |_|\__,_|___|_|  |_| ncurses-based, FAIM-based AIM client
**
** FAIM Copyright 1998 Adam Fritzler <afritz@iname.com>
** FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
*/
#include <naim.h>

WINDOW	*win_status = NULL,
	*win_input = NULL,
	*win_buddy = NULL;
extern buddywin_t	*curbwin;
extern faimconf_t	faimconf;
int	wsetup_called = 0;

int	wprintf(WINDOW *window, int pair, int bold, const char *format, ...) {
	va_list	msg;

	assert(window != NULL);
	assert(format != NULL);

	va_start(msg, format);
	wattrset(window, (bold?A_BOLD:0) | COLOR_PAIR(pair));
	vwprintw(window, format, msg);
	wattrset(window, 0);
	va_end(msg);
}

void	winitwin(WINDOW *window, int colorpair) {
	assert(window != NULL);

	idlok(window, TRUE);
	scrollok(window, TRUE);
	intrflush(window, FALSE);
	keypad(window, TRUE);
	meta(window, TRUE);
	nodelay(window, TRUE);
	leaveok(window, TRUE);
	notimeout(window, TRUE);
	wclear(window);
	wbkgd(window, COLOR_PAIR(colorpair));
}

int	wcols(faimconf_t *conf) {
	return(conf->cols = 66);
}

#define NWIN(WIN)	newwin(faimconf.WIN.widthy, faimconf.WIN.widthx, \
	faimconf.WIN.starty, faimconf.WIN.startx);
#define SETP(pair)	init_pair((pair), faimconf.colors[(pair)].fore, \
	faimconf.colors[(pair)].back);
void	wsetp(void) {
	SETP(IMWIN_BGCOLOR);
	SETP(IMWIN_TEXT_U);
	SETP(IMWIN_TEXT_R);
	SETP(IMWIN_STATUS);
	SETP(INPUT_BGCOLOR);
	SETP(INPUT_TEXT);
	SETP(INPUT_TEXT_ALT);
	SETP(BUDDY_BGCOLOR);
	SETP(BUDDY_TEXT);
	SETP(BUDDY_WAITING);
	SETP(BUDDY_OFFLINE);
	SETP(BUDDY_NONBUDDY);
	SETP(STAT_BGCOLOR);
	SETP(STAT_TEXT);
	SETP(BACKDROP);
}

void	wsetup(void) {
	if (wsetup_called > 0) {
		printf("wsetup() was just called, but it's already been" 
			" run!\n");
		return;
	}
	initscr();
	if (LINES < 20) {
		printf("You do not have enough rows in your current"
			" screen.\nI require at least 20, but you only"
			" have %i.\n", LINES);
		exit(0);
	}

	if (COLS < 20) {
		printf("You do not have enough columns in your current"
			" screen.\nI require at least 20, but you only"
			" have %i.\n", COLS);
		exit(0);
	}
	cbreak();
	noecho();
	nonl();
	nodelay(stdscr, TRUE);
	timeout(0);

	start_color();
	wsetp();

	curbwin = NULL;		// ahhhh, wsetup() assumes bsetup() has
				// been run, but bsetup() requires
				// wsetup() to already be run
	winitwin(stdscr, BACKDROP);
	wrefresh(stdscr);

	win_status = NWIN(wstatus);
	assert(win_status != NULL);
	winitwin(win_status, STAT_BGCOLOR);

	win_input = NWIN(winput);
	assert(win_input != NULL);
	winitwin(win_input, INPUT_BGCOLOR);

	win_buddy = NWIN(wbuddy);
	assert(win_buddy != NULL);
	winitwin(win_buddy, BUDDY_BGCOLOR);
	scrollok(win_buddy, FALSE);

	wsetup_called = 1;
}

void	wshutitdown(void) {
	if (wsetup_called == 0) {
		printf("wsetup() was just called, but it's already been" 
			" run!\n");
		return;
	}
	endwin();
	wsetup_called = 0;
}
